/* -*-c++-*-
 *Header:
 *File: file.h (for C++)
 *Author: Tom Howard
 *Date: 2001/9/31
 *EndHeader:
 */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */



#ifndef _FILE_H_
#define _FILE_H_

#include <strstream>
#include <iostream>
using namespace std;

#include "config.h"

#ifdef HAVE_LIBZ
#include <zlib.h>
#endif

#include <stdio.h>

#ifndef HAVE_LIBZ
#define Z_NO_COMPRESSION 	0x0000
#define Z_BEST_SPEED 		0x0001
#define Z_BEST_COMPRESSION	0x0009
#define Z_DEFAULT_COMPRESSION   (-0x0001)

#define Z_DEFAULT_STRATEGY	0x0000
#define Z_FILTERED	     	0x0001
#define Z_HUFFMAN_ONLY		0x0002
#endif 

class file
{
public:
  enum
  {
    DEFAULT_COMPRESSION = 	Z_DEFAULT_COMPRESSION,
    NO_COMPRESSION = 		Z_NO_COMPRESSION,
    BEST_SPEED = 		Z_BEST_SPEED,
    BEST_COMPRESSION = 		Z_BEST_COMPRESSION,
    COMPRESSION_MOD = 		0x0010, 
    DEFAULT_STRATEGY = 		Z_DEFAULT_STRATEGY * COMPRESSION_MOD,
    FILTERED =			Z_FILTERED * COMPRESSION_MOD,
    HUFFMAN_ONLY = 		Z_HUFFMAN_ONLY * COMPRESSION_MOD,
    READ = 			0x0100,
    WRITE = 			0x0200,
    BINARY = 			0x0800
  };

  file ();
  ~file ();

  void open ( char* path, int mode );
  void open ( int fd, int mode );
  
  void setCompression ( int level, int strategy );

  int read ( void* buf, unsigned len );
  int write ( void* buf, unsigned len );

  int flush ( int type = 0 );

  long seek ( long offset, int whence );
  long tell ();
  long rewind ();

  bool eof ();

  int close ();

  int compression ();
  bool filtered ();
  bool huffmanOnly ();
  bool binary ();
  bool read ();
  bool write ();

  bool open ();

  void clearErr ();
private:
  FILE* fp;
#ifdef HAVE_LIBZ
  gzFile gzfp;
#endif

  bool f_open;
  bool gz_open;
  int _mode;
};

#endif
