/* -*- Mode: C++ -*-
 *Header:
 *File: logplayer.C
 *Author: Noda Itsuki
 *Date: 1996/11/28
 *EndHeader:
 */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <sys/time.h>
#include <strstream.h>
#include <sched.h>
#include "param.h"
#include "types.h"
#include "logplayer.h"
#include "utility.h"

extern Player *Play ;
 
static void update(void) ;

Player::Player()
{

  m_plFilePos = new long [MAX_SHOWINFO];
  if (m_plFilePos == NULL)
    exit(-1);

  controler = new Controler();
  if (controler == NULL)
    exit(-1);
}

Player::~Player()
{
  delete controler;
  delete [] m_plFilePos;
}

void Player::init(int argc, char *argv[])
{
  GetOption(argc, argv) ;

  /* open input file */
  OpenForRead(in_file);	

  /* open output file */
  OpenForWrite(out_file);

  state = STATE_WAIT ;
  limit = 10000 ;
  sent = 0 ;
  rect = 0 ;
  to_time = NONE ;
  rec_state = REC_OFF ;
  current = 1;
  port.init() ;

  /* X logplayer */
  if (!no_window) {
    controler->init(argc, argv) ;
    timer_init() ;
    controler->display_status() ;
    controler->Loop() ;
  }
  /* no window logplayer */
  else {
    /* open command file */
    if (com_file[0] != '\0') {
      if ((comfp = fopen(com_file, "r")) == NULL) {
	cerr << __FILE__ << ": " << __LINE__ 
	     << ": Can't open log file " << com_file << endl ;
	infp.close ();
	if (out_file[0] != '\0')
	  fclose(outfp) ;
	exit(-1) ;
      }
    }
    nw_Init() ;
    nw_TimerInit() ;
    nw_Loop() ;
    nw_Quit() ;
  }
}

void Player::GetOption(int argc, char **argv)
{
  option_t opt[] = {
    {"out",					(void *)&(out_file),		V_STRING},
    {"port",				(void *)&(port.portnum),	V_INT},
    {"nw",					(void *)&(no_window),		V_NONE},
    {"nowindow",			(void *)&(no_window),		V_NONE},
    {"f",					(void *)&(com_file),		V_STRING},
    {"message_delay_interval",              (void *)&message_delay_interval, V_INT},
    {"\0",					NULL,						0}

  } ;

  char buf[MaxMesg] ;
  char com[256] ;
  char onoff[16] ;
  int i, p, n ;
  int flag = 0;
#ifdef HAVE_LIBZ
  gzFile gzfp;
#endif
  FILE *fp ;

  /* set default value */
  in_file[0] = '\0' ;
  out_file[0] = '\0' ;
  com_file[0] = '\0' ;
  port.portnum = DEFAULT_PORT_NUMBER ;
  no_window = false ;
  message_delay_interval = 20;
	
  /* skip command name */
  argv++ ; argc-- ;

  /* first, search option '-file' */
  for(i = 0 ; i < argc ; i++) {
    if (!strcmp(*(argv + i), "-file")) {
      if (i == argc - 1) {
	cerr << "can't find file name" << endl ;
	break ;
      }
      fp = fopen(*(argv+i+1), "r");
      if ( fp == NULL ) {
	cerr << "can't open config file " << *(argv+i+1) << endl ;
	break ;
      }

      while(fgets(buf,MaxMesg,fp) != NULL) {
				/* ignore remark line */
	if (buf[0] == '#' || buf[0] == '\n')
	  continue ;

				/* replace from ':' to ' ' */
	char *r = buf ;
	while(*r != '\0') {
	  if (*r == ':') *r = ' ' ;
	  r++ ;
	}

	n = sscanf(buf, "%s", com) ;
	if (n < 1) {
	  cerr << "Illegal line : " << buf ;
	  continue ;
	}

	for (p = 0 ; opt[p].vptr != NULL ; p++) {
	  if (strcmp(com, opt[p].optname))
	    continue ;

	  /* match */
	  switch(opt[p].vsize) {
	  case V_INT:
	    n = sscanf(buf, "%s %d", com, (int *)opt[p].vptr) ;
	    break ;

	  case V_DOUBLE:
	    n = sscanf(buf, "%s %lf", com, (double *)opt[p].vptr) ;
	    break ;

	  case V_STRING:
	    n = sscanf(buf, "%s %s", com, (char *)opt[p].vptr) ;
	    break ;

	  case V_ONOFF:
	    n = sscanf(buf, "%s %s", com, onoff) ;
	    *((int *)opt[p].vptr) = (!strcmp(onoff, "on")) ? 
	      true : false ;
	    break ;

	  case V_NONE:
	    *((int *)opt[p].vptr) = true ;
	    n = 2 ;
	    break ;
	  }

	  if (n > 2)
	    cerr << "Illegal line (" << com << ") " << endl ;

	  break ;
	}

	if (opt[p].vptr == NULL)
	  cerr << "Illegal line (" << com << ") " << endl ;
      }

      fclose(fp) ;
      break ;
    }
  }

  /* next, analyze command line option */
  while (argc) {
    if (!strcmp(*argv, "-file")) {
      argv += 2 ;
      argc -= 2 ;
      continue ;
    }

    if (strncmp(*argv, "-", 1)){
      if (!flag++)
	strcpy(in_file, *argv) ;
      else
	in_file[0] = '\0' ;
      argv++ ;
      argc-- ;
      continue ;
    }

    for (p = 0 ; opt[p].vptr != NULL ; p++) {
      if (strcmp(*argv + 1, opt[p].optname))
	continue ;

      /* match */
      argv++ ;
      argc-- ;

      switch(opt[p].vsize) {
      case V_INT:
	*((int *)opt[p].vptr) = atoi(*argv) ;
	break ;

      case V_DOUBLE:
	*((double *)opt[p].vptr) = atof(*argv) ;
	break ;

      case V_STRING:
	strcpy((char *)opt[p].vptr, *argv) ;
	break ;
				
      case V_ONOFF:
	strcpy(onoff, *argv) ;
	*((int *)opt[p].vptr) = (!strcmp(onoff, "on")) ? 
	  true : false ;
	break ;
      case V_NONE:
	*((int *)opt[p].vptr) = true ;
	argv-- ;
	argc++ ;
	break ;
      }

      break ;
    }

    if (opt[p].vptr == NULL)
      cerr << "Unrecognized Opion : " << *argv << endl ;

    argv++ ;
    argc-- ;
  }
}

void Player::Quit(void)
{
  if ( infp.close () )
    cerr << "Can't close input file" << endl ;

  if (out_file[0] != '\0') {
    fflush(outfp) ;
    if (fclose(outfp))
      cerr << "Can't close output file" << endl ;
  }

  exit(1) ;
}

void Player::timer_init(void)
{
#ifndef X11R5
  struct sigaction alarm_action ;
  struct itimerval itv ;
  void signal_handler(void) ;

  sig_id = XtAppAddSignal(controler->app_context,
			  (XtSignalCallbackProc)update, NULL) ;
  itv.it_interval.tv_sec = 0 ;
  itv.it_interval.tv_usec = TIMEDELTA * 1000 ;
  itv.it_value.tv_sec = 0 ;
  itv.it_value.tv_usec = TIMEDELTA * 1000 ;

#ifdef Linux_2_2	
  alarm_action.sa_handler = (void (*)(int))signal_handler ;
#else
  alarm_action.sa_handler = (void (*)(...))signal_handler ;
#endif	
  alarm_action.sa_flags = 0;        /* init */              // [2000/11/20.frehberg.cs.tu-berlin.de]

  sigaction(SIGALRM, &alarm_action, NULL) ;
  setitimer(ITIMER_REAL, &itv, NULL) ;

#else
  XtIntervalId intid ;
  intid = XtAppAddTimeOut(controler->app_context,
			  TIMEDELTA, (XtTimerCallbackProc)update, NULL) ;
#endif

}


void Player::read_file(void) 
{
  // read until a new showinfo is reached
  m_bReadShowInfo = 0;
  while (m_bReadShowInfo == 0) {
    if (ReadLog() == 0) { 
      StopLog();
      return ;
    }

    SendLog();
    WriteLog();
  }

  if (current == to_time)		
    StopLog();
}

void Player::rewind_file(void) 
{
  // is there a cycle to rewind to ?
  if ((current <= 1) 
      || (infp.seek( m_plFilePos[current-1], SEEK_SET) == -1)
      ) {
    infp.clearErr () ;
    RewindLog() ;
    StopLog();
    return ;
  }

  if (ReadLog() == 0){
    StopLog();
    return ;
  }

  SendLog();
  WriteLog();

  if (current == to_time)
    StopLog();
}

void Player::jump(void) 
{
  long offset ;
  offset = infp.tell() ;

  if (to_time == END) {
    infp.seek( m_plFilePos[m_iMaxPos], SEEK_SET) ;
    for ( ; ; ) {
      // read to the end of the file
      if (ReadLog() == 0) {
	infp.seek( m_plFilePos[m_iMaxPos], SEEK_SET) ;
	StopLog();
	state = STATE_NOT_JUMP ;
	return ;
      }
    }
  }
  else if (to_time <= m_iMaxPos) {
    // just jump to the position
    infp.seek(m_plFilePos[to_time], SEEK_SET);
    if (ReadLog() == 0) {
      infp.clearErr() ;
      RewindLog() ;
      StopLog();
    }
  }
  else {		
    // we did not read so far up to now
    infp.seek( m_plFilePos[m_iMaxPos], SEEK_SET) ;
    for ( ; ; ) {
      if (ReadLog() == 0) {
	infp.clearErr() ;
	RewindLog() ;
	StopLog();
	return;
      }
      else if (current == to_time) {
	to_time = NONE ;
	break ;
      }
    }
  }

  SendLog();
  WriteLog();
  StopLog();
}

void Player::send_blank(void) 
{
  m_tBuf.mode = htons(BLANK_MODE) ;
  controler->display_time(ntohs((unsigned short)-1)) ;

  SendLog();
  WriteLog();
}

static void update(void)
{
#ifdef X11R5
  XtIntervalId intid ;
#endif

  Play->sent += TIMEDELTA ;
  Play->rect += TIMEDELTA ;

  if (Play->sent >= Play->limit ) {
    switch (Play->state) {
    case STATE_PLAY:
      Play->limit = PLAY_CYCLE ;
      Play->read_file() ;
      break ;

    case STATE_STOP:
      Play->limit = PLAY_CYCLE ;
      break ;

    case STATE_REVERSE:
      Play->limit = PLAY_CYCLE ;
      Play->rewind_file() ;
      break ;

    case STATE_REW:
      Play->limit = FEED_CYCLE ;
      Play->rewind_file() ;
      break ;

    case STATE_FOR:
      Play->limit = FEED_CYCLE ;
      Play->read_file() ;
      break ;

    case STATE_PLUS:
      Play->limit = 10 ;
      Play->to_time = Play->current + 1 ;
      Play->read_file() ;
      break ;

    case STATE_MINUS:
      Play->limit = 10;
      Play->to_time = Play->current - 1 ;
      Play->rewind_file() ;
      break ;

    case STATE_BLANK:
      Play->limit = PLAY_CYCLE ;
      Play->send_blank() ;
      break ;

    case STATE_JUMP:
      Play->limit = 10 ;
      Play->jump() ;
      break ;

    default:
      break ;
    }

    Play->sent = 0 ;
  }
  if (Play->rect >= 400) {
    Play->port.recv_info() ;
    if(Play->state == STATE_WAIT) {
      if(strcmp(Play->port.rbuf, "(dispinit)") && strcmp(Play->port.rbuf,  "(dispinit version 2)") )
	{
	  Play->sent = 0 ;		
	}
      else {
	Play->limit = 100 ;
	Play->state = STATE_STOP ;
	Play->controler->display_status() ;
      }
    }
    Play->rect = 0 ;
  }
#ifdef X11R5
  intid = XtAppAddTimeOut(Play->controler->app_context,
			  TIMEDELTA, (XtTimerCallbackProc)update, NULL) ;
#endif
}

#ifndef X11R5
void signal_handler(void)
{
  XtNoticeSignal(Play->sig_id) ;
}
#endif

// added 16.04.98 (Klaus Dorer)
// I think this should reduce logfile sizes by a factor of 3 to 7

int Player::ReadLog(void)
{
  short		len;
  long		pos;
  Value		tmp_val;
  Angle		tmp_ang;
  long		tmp_long;

  pos  = infp.tell();
  // m_iVersion may contain the version information of the logfile to be
  // able to read and write older versions
  switch (m_iVersion)
    {
    case REC_OLD_VERSION:
      // it is a logfile of the previous version
      if (infp.read( (void *) (&m_tBuf), sizeof(dispinfo_t)) == 0)
      if (ntohs(m_tBuf.mode) == SHOW_MODE) 
	NewShowinfo(pos);
      else
	m_bReadShowInfo = 0;
      return true;

    case REC_VERSION_2:
      // new version 
      // read the type of next msg
      if (infp.read((void *) (&m_tBuf.mode), sizeof(m_tBuf.mode)) == 0)
	return false;
      m_tBuf2.mode = m_tBuf.mode;
      switch (ntohs(m_tBuf.mode))
	{
	case SHOW_MODE:
	  // read a show struct
	  if (infp.read((void *) (&m_tBuf.body.show), 
		     sizeof(m_tBuf.body.show)) == 0)
	    return false;

	  m_tBuf2.body.show.pmode = m_tBuf.body.show.pmode;
	  m_tBuf2.body.show.team = m_tBuf.body.show.team;			
	  m_tBuf2.body.show.ball.x = nstonl ( m_tBuf.body.show.pos [ 0 ].x );
	  m_tBuf2.body.show.ball.y = nstonl ( m_tBuf.body.show.pos [ 0 ].y );

	  for ( int i = 0; i < MAX_PLAYER * 2; i++ )
	    {
	      m_tBuf2.body.show.pos [ i ].mode = m_tBuf.body.show.pos [ i+1 ].enable;
	      m_tBuf2.body.show.pos [ i ].body_angle = htonl((long)(Deg2Rad( (double)(short)ntohs( m_tBuf.body.show.pos [ i+1 ].angle ) ) * SHOWINFO_SCALE2));
	      m_tBuf2.body.show.pos [ i ].x = nstonl ( m_tBuf.body.show.pos [ i+1 ].x );
	      m_tBuf2.body.show.pos [ i ].y = nstonl ( m_tBuf.body.show.pos [ i+1 ].y );			    
	    }
	  m_tBuf2.body.show.time = m_tBuf.body.show.time;
			
			
	  NewShowinfo(pos);
	  return true;

	case MSG_MODE:
	  // it is a message
	  m_bReadShowInfo = 0;
	  // read the board info
	  if (infp.read((void *) (&m_tBuf.body.msg.board), 
		     sizeof(m_tBuf.body.msg.board)) == 0)
	    return 0;

	  m_tBuf2.body.msg.board = m_tBuf.body.msg.board;
	  // read the string length
	  if (infp.read((void *) (&len), 
		     sizeof(len)) == 0)
	    return 0;
	  len = ntohs(len);
		
	  // read the message
	  if (infp.read((void *) (m_tBuf.body.msg.message), 
		     len) == 0)
	    return 0;
	  m_tBuf2.body.msg.message = m_tBuf.body.msg.message;
	  return 1;		  
	}
    case REC_VERSION_3:
      if (infp.read((void *) (&m_tBuf2.mode), 
		 sizeof(m_tBuf2.mode)) == 0)
	return false;


      switch (ntohs(m_tBuf2.mode))
	{
	case PM_MODE:
	  if (infp.read((void *) (&m_playmode_buf), 
		     sizeof(m_playmode_buf)) == 0)
	    return false;

	  return ReadLog();

	case TEAM_MODE:
	  if (infp.read((void *) (m_team_buf), 
			sizeof(team_t)*2) == 0)
	    return false;

	  return ReadLog();
		  
		  
	case SHOW_MODE:

	  m_tBuf.mode = m_tBuf2.mode;

	  m_tBuf.body.show.pmode = m_playmode_buf;
	  m_tBuf2.body.show.pmode = m_playmode_buf;
	  m_tBuf.body.show.team[0] = m_team_buf[0];
	  m_tBuf.body.show.team[1] = m_team_buf[1];
	  m_tBuf2.body.show.team[0] = m_team_buf[0];
	  m_tBuf2.body.show.team[1] = m_team_buf[1];
			
	  // read a show struct
	  if (infp.read((void *) (&m_show_buf), 
		     sizeof(m_show_buf)) == 0)
	    return false;
		
	  m_tBuf2.body.show.ball = m_show_buf.ball;

	  m_tBuf.body.show.pos [ 0 ].x = nltons (m_show_buf.ball.x);
	  m_tBuf.body.show.pos [ 0 ].y = nltons (m_show_buf.ball.y);
			
	  for ( int i = 0; i < MAX_PLAYER * 2; i++ )
	    {
	      m_tBuf2.body.show.pos [ i ] = m_show_buf.pos [ i ];
	      m_tBuf.body.show.pos [ i+1 ].enable = m_show_buf.pos [ i ].mode;

	      m_tBuf.body.show.pos [ i+1 ].angle = htons((short)(Rad2Deg( (double)(long)ntohl( m_show_buf.pos [ i ].body_angle ) ) / SHOWINFO_SCALE2) );
	      m_tBuf.body.show.pos [ i+1 ].x = nltons ( m_show_buf.pos [ i ].x );
	      m_tBuf.body.show.pos [ i+1 ].y = nltons ( m_show_buf.pos [ i ].y );
	      m_tBuf.body.show.pos [ i+1 ].unum = htons((i % MAX_PLAYER) + 1); 
	      m_tBuf.body.show.pos [ i+1 ].side = htons(((i < MAX_PLAYER) ? LEFT : RIGHT )); 
	    }
			
	  m_tBuf2.body.show.time = m_show_buf.time;
	  m_tBuf.body.show.time = m_show_buf.time;

	  NewShowinfo(pos);
	  return true;

	case MSG_MODE:
	  // it is a message
	  m_bReadShowInfo = 0;
	  // read the board info
	  if (infp.read((void *) (&m_tBuf.body.msg.board), 
		     sizeof(m_tBuf.body.msg.board)) == 0)
	    return false;
	  m_tBuf2.body.msg.board = m_tBuf.body.msg.board;
	  // read the string length
	  if (infp.read((void *) (&len), 
		     sizeof(len)) == 0)
	    return false;

	  len = ntohs(len);
		
	  // read the message
	  if (infp.read((void *) (m_tBuf.body.msg.message), 
		     len) == 0)
	    return false;
	
	  m_tBuf2.body.msg.message = m_tBuf.body.msg.message;
	  

	  return true;
	case PARAM_MODE:
	  {
	    if (infp.read((void *) (&m_tBuf2.body.sparams), 
		     sizeof(m_tBuf2.body.sparams)) == 0)
	      return false;
	    return true;
	  }
	case PPARAM_MODE:
	  {
	    if (infp.read((void *) (&m_tBuf2.body.pparams), 
		       sizeof(m_tBuf2.body.pparams)) == 0)
		return false;
	    return true;
	  }
	case PT_MODE:
	  {
	    if (infp.read((void *) (&m_tBuf2.body.ptinfo), 
		       sizeof(m_tBuf2.body.ptinfo)) == 0)
	      return false;

	    return true;
	  }
		  
	}
	  

    }
}

void Player::WriteLog(void)
{
  short		len = 1;

  // recording ?
  if(rec_state != REC_ON)
    return;

  if ( m_iVersion == REC_VERSION_3 )
    {
      static char pm = (char)0;
      static team_t teams[2] = { { "", 0 },
				 { "", 0 } };


      // if playmode has changed wirte playmode
      if ( pm != m_tBuf2.body.show.pmode )
	{
	  pm = m_tBuf2.body.show.pmode;
	  short mode = htons(PM_MODE);
	  fwrite((char *)(&mode), sizeof(mode), 1, outfp) ;
	  fwrite((char *)(&pm), sizeof(pm), 1, outfp) ;      

	}

      // if teams or score has changed, write teams and score
      if ( strcmp ( teams [ 0 ].name, m_tBuf2.body.show.team [ 0 ].name ) ||
	   teams [ 0 ].score != m_tBuf2.body.show.team [ 0 ].score ||
	   strcmp ( teams [ 1 ].name, m_tBuf2.body.show.team [ 1 ].name ) ||
	   teams [ 1 ].score != m_tBuf2.body.show.team [ 1 ].score
	   )
	{
	  strcpy ( teams [ 0 ].name, m_tBuf2.body.show.team [ 0 ].name );
	  teams [ 0 ].score = m_tBuf2.body.show.team [ 0 ].score;
	  strcpy ( teams [ 1 ].name, m_tBuf2.body.show.team [ 1 ].name );
	  teams [ 1 ].score = m_tBuf2.body.show.team [ 1 ].score;

	  short mode = htons(TEAM_MODE);
	  fwrite((char *)(&mode), sizeof(mode), 1, outfp) ;
	  fwrite((char *)(teams), sizeof(team_t), 2, outfp) ;      

	}

      // write positional data
      short_showinfo_t2 pos_data;
      pos_data.ball = m_tBuf2.body.show.ball;
      for ( int i = 0; i < MAX_PLAYER * 2; i++ )
	pos_data.pos [ i ] = m_tBuf2.body.show.pos [ i ];
      pos_data.time = m_tBuf2.body.show.time;

      short mode = htons(SHOW_MODE);
      fwrite((char *)(&mode), sizeof(mode), 1, outfp) ;
      fwrite((char *)(&pos_data), sizeof(pos_data), 1, outfp) ;

  
      //fflush(outfp) ;
    }
  else
    {
      // write the type of next msg
      fwrite((char *)(&m_tBuf.mode), sizeof(m_tBuf.mode), 1, outfp);

      switch (ntohs(m_tBuf.mode))
       	{
       	case SHOW_MODE:
	  // write a show struct
	  fwrite((char *)(&m_tBuf.body.show), sizeof (m_tBuf.body.show), 1, outfp);
	  break;

       	case MSG_MODE:
	  // it is a message
	  // write the board info
	  fwrite((char *)(&m_tBuf.body.msg.board), sizeof(m_tBuf.body.msg.board), 1, outfp);

	  // calculate the string length and write it
	  while ((m_tBuf.body.msg.message[len-1] != '\0') && (len < 2048))
	    len++;
	
	  short nlen = htons(len); 
	  fwrite((char *)(&nlen), sizeof(nlen), 1, outfp);

	  // write the message
	  fwrite(m_tBuf.body.msg.message, len, 1, outfp);
	  break;
       	}
    }
}

long Player::sleepByMilliSeconds(long msecs)
{
  long timeDelay = 0;
  timeval tp_start, tp_run;
  gettimeofday ( &tp_start, NULL );

  while ( timeDelay < msecs ) {
    gettimeofday ( &tp_run, NULL );
    timeDelay = (tp_run.tv_sec - tp_start.tv_sec) * 1000 +
      (tp_run.tv_usec - tp_start.tv_usec) / 1000;
    sched_yield ();
  }
  return timeDelay;
}

void Player::SendLog(void)
{
  displist_t *p ;

  /* we keep track of how many messages we have sent and delay periodically to
     give the monitor a chance to keep up. With UDP there's no way to insure that
     we don't lose packets, but this is a bit better experimentally */
  static int num_sent = 0;
  if (++num_sent > message_delay_interval  && message_delay_interval >= 0 ) {
    num_sent = 0;
    sleepByMilliSeconds( 10 );
  }
  
  for (p = port.top.next; p != NULL; p = p->next)
    {
      if ( p->version == 1 )
	{
	  switch (m_iVersion)
	    {
	    case REC_OLD_VERSION:
	    case REC_VERSION_2:
	      port.send_info(&m_tBuf, p->shost, p->sport) ;
	      break;
	  
	    case REC_VERSION_3:
	      if (ntohs(m_tBuf2.mode) == SHOW_MODE) {
		port.send_info(&m_tBuf, p->shost, p->sport) ;
	      }
	      break;
	default:
	  cerr << "What is m_iVersion? " << m_iVersion
	       << endl;
	  break;
	    }
	}

      else {
#ifdef DEBUG_CODE
	switch (ntohs(m_tBuf2.mode)) {
	case NO_INFO:
	  cerr << "Why sending no info?" << endl;
	  break;
	case SHOW_MODE:
	  cout << "Sending show mode; time: " << ntohs(m_tBuf2.body.show.time) << endl;
	  break;
	case MSG_MODE:
	  cout << "Sending msg mode" << endl;
	  break;
	case DRAW_MODE: break;
	case BLANK_MODE: break;
	case PM_MODE: break;
	case TEAM_MODE: break;
	case PT_MODE: break;
	case PARAM_MODE: break;
	case PPARAM_MODE: break;
	  break;
	default:
	  cerr << "What is mode? " << ntohs(m_tBuf2.mode) << endl;
	}
#endif
	port.send_info(&m_tBuf2, p->shost, p->sport) ;
      }
    }
}

void Player::RewindLog(void)
{
  switch (m_iVersion)
    {
    case REC_OLD_VERSION:
      infp.rewind ();
      break;

    case REC_VERSION_2:
    case REC_VERSION_3:
      // there are 4 bytes at the beginning specifying the version,
      // so overread
      infp.seek( 4L, SEEK_SET);
      break;
    }
}

void Player::StopLog(void)
{
  state = STATE_STOP ;
  to_time = NONE ;
  if (!no_window) {
    controler->display_time(current) ;
    controler->display_status() ;
    controler->buttons_reset() ;
  }	
}

void Player::OpenForRead(char* in_file)
{
  char localbuf[6];

  // check if an input file is specified
  if (in_file[0] == '\0') {
    cerr << "Usage: logplayer file_name\n\t\t[-out file_name]\n\t\t";
    cerr << "[-port num]\n\t\t[-nw]\n\t\t[-file file_name]\n" << endl ;
    exit(-1) ;
  }

  // open the file
  infp.open(in_file, file::READ | file::BINARY );
  if ( !infp.open () ) {
    cerr << "open = " << infp.open () << endl; 
    cerr << __FILE__ << ": " << __LINE__ 
	 << ": Can't open log file " << in_file << endl ;
    exit(-1) ;
  }

  // default is old version
  m_iVersion = REC_OLD_VERSION;
  // have not read anything from file up to now
  m_iMaxPos = 0;
  // read version information
  if ( infp.read( localbuf, 4) == 0 )
    {
      // could not read from file
      infp.close ();
      cerr << __FILE__ << ": " << __LINE__
	   << ": Can't open log file " << in_file << endl ;
      exit(-1) ;
    }

  // check version
  if ((localbuf[0] == 'U') && (localbuf[1] == 'L') && (localbuf[2] == 'G'))
    {
      m_iVersion = localbuf[3];
      cout << "Logfile version: " << m_iVersion << endl;
    }
  else
    {
    // no version information, so it is an old logfile
    // (or no logfile at all)
      cout << "Logfile version: " << REC_OLD_VERSION << endl;
      infp.rewind ();
    }
}

void Player::OpenForWrite(char* out_file)
{
  char	buf[5];
  int	append = false;

  if (out_file[0] == '\0') 
    return;

  // check if we append or create a new file, because version information
  // must not be written when appending
  if ((outfp = fopen(out_file, "rb")) != NULL)
    {
      append = true;
      fclose(outfp);
    }
		
  // open the file
  if ((outfp = fopen(out_file, "ab")) == NULL){
    cerr << "Can't open output file " << out_file << endl ;
    infp.close ();
    exit(-1) ;
  }

  // write version information
  if (append != true)
    {
      buf[0] = 'U';
      buf[1] = 'L';
      buf[2] = 'G';
      buf[3] = m_iVersion;
      fwrite(buf, 4, 1, outfp);
    }
}

void Player::NewShowinfo(long pos)
{
  int	i;

  current = ntohs(m_tBuf.body.show.time);
  if (current < 0) {
    cerr << "strange time\n";
    current = 0;
    return;
  }
	  
  if (!no_window)
    controler->display_time(current) ;
  // test, if this info has never been read before
  if (current > m_iMaxPos) {
    if (current < MAX_SHOWINFO) {
      for (i = m_iMaxPos + 1; i <= current; i++)
	// just for the case, that timesteps are missing
	m_plFilePos[i] = pos;
      m_iMaxPos = current;
    }
  } 		
  m_bReadShowInfo = 1;
}



