/*
 *Header:
 *File: logplayer.h (for C++)
 *Author: Noda Itsuki
 *Date: 1996/11/28
 *EndHeader:
 */

/*
 *Copyright:

    Copyright (C) 1996-2000 Electrotechnical Laboratory. 
    	Itsuki Noda, Yasuo Kuniyoshi and Hitoshi Matsubara.
    Copyright (C) 2000, 2001 RoboCup Soccer Server Maintainance Group.
    	Patrick Riley, Tom Howard, Daniel Polani, Itsuki Noda,
	Mikhail Prokopenko, Jan Wendler 

    This file is a part of SoccerServer.

    This code is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *EndCopyright:
 */



#include "stdio.h"
#include "netif.h"
#include <X11/Intrinsic.h>
#include "file.h"

#define PLAY_CYCLE		150 		/* [msec] */
#define FEED_CYCLE		50
#define STEP_CYCLE		450

#define STATE_WAIT		-1
#define STATE_STOP		0
#define STATE_PLAY		1
#define STATE_REVERSE	2	
#define STATE_REW		3
#define STATE_FOR		4
#define STATE_PLUS		5
#define STATE_MINUS		6
#define STATE_BLANK		7
#define STATE_JUMP		8
#define STATE_NOT_JUMP	9

#define	REC_OFF		0
#define	REC_ON		1

#define END			-1
#define FOREVER		-2
#define NONE		-3

// added 16.04.98
// logfile format versions
#define MAX_SHOWINFO	10000

/*
 *===================================================================
 *Part: Controler Class
 *===================================================================
 */
class Controler {
	public:
		XtAppContext	app_context ;			/* application context */
		Widget			top ;					/* top widget */
		Widget			main ;					/* main widget */
		Widget			status ;				/* status widget */
		Widget			buttons ;				/* buttons widget */
		Widget			time ;					/* time widget */
		Widget			quit ;
		Widget			rew ;
		Widget			rev ;
		Widget			minus ;
		Widget			stop ;
		Widget			plus ;
		Widget			play ;
		Widget			ff ;
		Widget			record_buttons ;		/* record buttons widget */
		Widget			rec ;					/* record button widget */
		Widget			blank ;					/* record button widget */
		Widget			jump ;
		Widget			input ;
		Widget			go ;
		Widget			speed ;

		void			init(int argc, char **argv) ;
		void			assign(void) ;
		void			display_time(int) ;
		void			display_status(void) ;
		void			change_recmode(void) ;
		void			buttons_reset(void) ;
		void			rew_button_reset_true(void) ;
		void			rew_button_reset_false(void) ;
		void			rev_button_reset_true(void) ;
		void			rev_button_reset_false(void) ;
		void			minus_button_reset(void) ;
		void			stop_button_reset_true(void) ;
		void			stop_button_reset_false(void) ;
		void			plus_button_reset(void) ;
		void			play_button_reset_true(void) ;
		void			play_button_reset_false(void) ;
		void			for_button_reset_true(void) ;
		void			for_button_reset_false(void) ;
		void			blank_button_reset(void) ;
		void			Loop(void) ;
} ;



/*
 *===================================================================
 *Part: Player Class
 *===================================================================
 */
class Player {
	public:
		Player();
		~Player();
		char			in_file[MAX_FILE_LEN] ;	/* input file name */
		char			out_file[MAX_FILE_LEN] ;	/* output file name */
		char			com_file[MAX_FILE_LEN] ;	/* command file name */
		int                     message_delay_interval ; /* how many messages to send to monitor before delaying */
		file			infp;
//		FILE			*infp ;	/* input file pointer */

		FILE			*outfp ;					/* output file pointer */
		FILE			*comfp ;					/* command file pointer */
		int				send_st ;					/* udp send step interval */
		int				sent ;						/* send counter timer */
		int				rect ;						/* recv counter timer */
		int				limit ;						/* loop time */
		int				state ;						/* status */
		int				current ;					/* current time */
		int				to_time ;					/* jump to [to_time] */
		int				rec_state ;					/* recording state */
		int				no_window ;					/* no window? */
		Port			port ;						/* communication port */
		Controler*		controler ;					/* controler interface */

#ifndef X11R5
		XtSignalId		sig_id ;					/* signal handler id */
#endif


		void			init(int argc, char **argv) ;
		void			timer_init(void) ;
		void			GetOption(int argc, char **argv) ;
		void			read_file(void) ;
		void			rewind_file(void) ;
		void			jump(void) ;
		void			send_blank(void) ;
		void			Quit(void) ;
		void			nw_Init(void) ;
		void			nw_TimerInit(void) ;
		void			nw_Loop(void) ;
		void			nw_Quit(void) ;
		void			nw_Play(void) ;
		void			nw_Rewind(void) ;
		void			nw_Sleep(void) ;

		// added 16.04.98
		int				ReadLog(void);
		void			WriteLog(void);
		void			SendLog(void);
		void			RewindLog(void);
		void			StopLog(void);
		void			OpenForRead(char* in_file);
		void			OpenForWrite(char* out_file);

	private:
		void			NewShowinfo(long pos);
                long                    sleepByMilliSeconds(long msecs);

		int				m_iMaxPos;			/* top position read from file */
		long*			m_plFilePos;		/* positions of showinfo in file */
		int				m_iVersion;			/* version of the file format */
		int				m_bReadShowInfo;	/* true,if showinfo has been read */
		dispinfo_t		m_tBuf;				/* buffer for dispinof */
		dispinfo_t2		m_tBuf2;
		short_showinfo_t2	m_show_buf;
		team_t			m_team_buf[2];
		char			m_playmode_buf;
} ;
